@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <glib.h>
 #ifndef _WIN32
@@ -26,6 +26,7 @@
 #define UNICODE
 #include <windows.h>
 #include <lm.h>
+#include "dns_sd_proxy.h"
 #endif
 
 #include "internal.h"
@@ -99,6 +100,17 @@ bonjour_login(PurpleAccount *account)
 	PurpleStatus *status;
 	PurplePresence *presence;
 
+#ifdef _WIN32
+	if (!dns_sd_available()) {
+		gc->wants_to_die = TRUE;
+		purple_connection_error(gc,
+			_("The Apple Bonjour For Windows toolkit wasn't found, see the FAQ at: "
+			  "http://developer.pidgin.im/wiki/Using%20Pidgin#CanIusePidginforBonjourLink-LocalMessaging"
+			  " for more information."));
+		return;
+	}
+#endif
+
 	gc->flags |= PURPLE_CONNECTION_HTML;
 	gc->proto_data = bd = g_new0(BonjourData, 1);
 
@@ -138,6 +150,8 @@ bonjour_login(PurpleAccount *account)
 		return;
 	}
 
+	bonjour_dns_sd_update_buddy_icon(bd->dns_sd_data);
+
 	/* Create a group for bonjour buddies */
 	bonjour_group = purple_group_new(BONJOUR_GROUP_NAME);
 	purple_blist_add_group(bonjour_group, NULL);
@@ -153,13 +167,13 @@ bonjour_close(PurpleConnection *connection)
 	BonjourData *bd = connection->proto_data;
 
 	/* Stop looking for buddies in the LAN */
-	if (bd->dns_sd_data != NULL)
+	if (bd != NULL && bd->dns_sd_data != NULL)
 	{
 		bonjour_dns_sd_stop(bd->dns_sd_data);
 		bonjour_dns_sd_free(bd->dns_sd_data);
 	}
 
-	if (bd->jabber_data != NULL)
+	if (bd != NULL && bd->jabber_data != NULL)
 	{
 		/* Stop waiting for conversations */
 		bonjour_jabber_stop(bd->jabber_data);
@@ -174,6 +188,8 @@ bonjour_close(PurpleConnection *connection)
 	if (bonjour_group != NULL)
 		purple_blist_remove_group(bonjour_group);
 
+	g_free(bd);
+	connection->proto_data = NULL;
 }
 
 static const char *
@@ -233,6 +249,13 @@ bonjour_set_status(PurpleAccount *account, PurpleStatus *status)
 	g_free(stripped);
 }
 
+static void bonjour_remove_buddy(PurpleConnection *pc, PurpleBuddy *buddy, PurpleGroup *group) {
+	if (buddy->proto_data) {
+		bonjour_buddy_delete(buddy->proto_data);
+		buddy->proto_data = NULL;
+	}
+}
+
 static GList *
 bonjour_status_types(PurpleAccount *account)
 {
@@ -283,17 +306,33 @@ bonjour_convo_closed(PurpleConnection *connection, const char *who)
 	bb->conversation = NULL;
 }
 
+static
+void bonjour_set_buddy_icon(PurpleConnection *conn, PurpleStoredImage *img)
+{
+	BonjourData *bd = conn->proto_data;
+	bonjour_dns_sd_update_buddy_icon(bd->dns_sd_data);
+}
+
+
 static char *
 bonjour_status_text(PurpleBuddy *buddy)
 {
-	PurplePresence *presence;
+	const PurplePresence *presence;
+	const PurpleStatus *status;
+	const char *message;
+	gchar *ret = NULL;
 
 	presence = purple_buddy_get_presence(buddy);
+	status = purple_presence_get_active_status(presence);
 
-	if (purple_presence_is_online(presence) && !purple_presence_is_available(presence))
-		return g_strdup(_("Away"));
+	message = purple_status_get_attr_string(status, "message");
 
-	return NULL;
+	if (message != NULL) {
+		ret = g_markup_escape_text(message, -1);
+		purple_util_chrreplace(ret, '\n', ' ');
+	}
+
+	return ret;
 }
 
 static void
@@ -301,6 +340,7 @@ bonjour_tooltip_text(PurpleBuddy *buddy, PurpleNotifyUserInfo *user_info, gboole
 {
 	PurplePresence *presence;
 	PurpleStatus *status;
+	BonjourBuddy *bb = buddy->proto_data;
 	const char *status_description;
 	const char *message;
 
@@ -318,6 +358,23 @@ bonjour_tooltip_text(PurpleBuddy *buddy, PurpleNotifyUserInfo *user_info, gboole
 	purple_notify_user_info_add_pair(user_info, _("Status"), status_description);
 	if (message != NULL)
 		purple_notify_user_info_add_pair(user_info, _("Message"), message);
+
+	/* Only show first/last name if there is a nickname set (to avoid duplication) */
+	if (bb->nick != NULL) {
+		if (bb->first != NULL)
+			purple_notify_user_info_add_pair(user_info, _("First name"), bb->first);
+		if (bb->first != NULL)
+			purple_notify_user_info_add_pair(user_info, _("Last name"), bb->last);
+	}
+
+	if (bb->email != NULL)
+		purple_notify_user_info_add_pair(user_info, _("E-Mail"), bb->email);
+
+	if (bb->AIM != NULL)
+		purple_notify_user_info_add_pair(user_info, _("AIM Account"), bb->AIM);
+
+	if (bb->jid!= NULL)
+		purple_notify_user_info_add_pair(user_info, _("XMPP Account"), bb->jid);
 }
 
 static gboolean
@@ -339,10 +396,9 @@ static PurplePluginProtocolInfo prpl_info =
 	OPT_PROTO_NO_PASSWORD,
 	NULL,                                                    /* user_splits */
 	NULL,                                                    /* protocol_options */
-	/* {"png", 0, 0, 96, 96, 0, PURPLE_ICON_SCALE_DISPLAY}, */ /* icon_spec */
-	NO_BUDDY_ICONS, /* not yet */                            /* icon_spec */
+	{"png,gif,jpeg", 0, 0, 96, 96, 65535, PURPLE_ICON_SCALE_DISPLAY}, /* icon_spec */
 	bonjour_list_icon,                                       /* list_icon */
-	NULL,													 /* list_emblem */
+	NULL,                                                    /* list_emblem */
 	bonjour_status_text,                                     /* status_text */
 	bonjour_tooltip_text,                                    /* tooltip_text */
 	bonjour_status_types,                                    /* status_types */
@@ -360,7 +416,7 @@ static PurplePluginProtocolInfo prpl_info =
 	NULL,                                                    /* change_passwd */
 	NULL,                                                    /* add_buddy */
 	NULL,                                                    /* add_buddies */
-	NULL,                                                    /* remove_buddy */
+	bonjour_remove_buddy,                                    /* remove_buddy */
 	NULL,                                                    /* remove_buddies */
 	NULL,                                                    /* add_permit */
 	NULL,                                                    /* add_deny */
@@ -384,7 +440,7 @@ static PurplePluginProtocolInfo prpl_info =
 	NULL,                                                    /* buddy_free */
 	bonjour_convo_closed,                                    /* convo_closed */
 	NULL,                                                    /* normalize */
-	NULL,                                                    /* set_buddy_icon */
+	bonjour_set_buddy_icon,                                  /* set_buddy_icon */
 	NULL,                                                    /* remove_group */
 	NULL,                                                    /* get_cb_real_name */
 	NULL,                                                    /* set_chat_topic */
@@ -412,11 +468,11 @@ static PurplePluginInfo info =
 	PURPLE_PLUGIN_MAGIC,
 	PURPLE_MAJOR_VERSION,
 	PURPLE_MINOR_VERSION,
-	PURPLE_PLUGIN_PROTOCOL,                             /**< type           */
+	PURPLE_PLUGIN_PROTOCOL,                           /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
 	NULL,                                             /**< dependencies   */
-	PURPLE_PRIORITY_DEFAULT,                            /**< priority       */
+	PURPLE_PRIORITY_DEFAULT,                          /**< priority       */
 
 	"prpl-bonjour",                                   /**< id             */
 	"Bonjour",                                        /**< name           */
@@ -426,10 +482,10 @@ static PurplePluginInfo info =
 	                                                  /**  description    */
 	N_("Bonjour Protocol Plugin"),
 	NULL,                                             /**< author         */
-	PURPLE_WEBSITE,                                     /**< homepage       */
+	PURPLE_WEBSITE,                                   /**< homepage       */
 
 	NULL,                                             /**< load           */
-	plugin_unload,                                             /**< unload         */
+	plugin_unload,                                    /**< unload         */
 	NULL,                                             /**< destroy        */
 
 	NULL,                                             /**< ui_info        */
@@ -444,18 +500,104 @@ static PurplePluginInfo info =
 	NULL
 };
 
+#ifdef WIN32
+static gboolean _set_default_name_cb(gpointer data) {
+	gchar *fullname = data;
+	const char *splitpoint;
+	GList *tmp = prpl_info.protocol_options;
+	PurpleAccountOption *option;
+
+	if (!fullname) {
+		purple_debug_info("bonjour", "Unable to look up First and Last name or Username from system; using defaults.\n");
+		return FALSE;
+	}
+
+	g_free(default_firstname);
+	g_free(default_lastname);
+
+	/* Split the real name into a first and last name */
+	splitpoint = strchr(fullname, ' ');
+	if (splitpoint != NULL) {
+		default_firstname = g_strndup(fullname, splitpoint - fullname);
+		default_lastname = g_strdup(&splitpoint[1]);
+	} else {
+		default_firstname = g_strdup(fullname);
+		default_lastname = g_strdup("");
+	}
+	g_free(fullname);
+
+
+	for(; tmp != NULL; tmp = tmp->next) {
+		option = tmp->data;
+		if (strcmp("first", purple_account_option_get_setting(option)) == 0)
+			purple_account_option_set_default_string(option, default_firstname);
+		else if (strcmp("last", purple_account_option_get_setting(option)) == 0)
+			purple_account_option_set_default_string(option, default_lastname);
+	}
+
+	return FALSE;
+}
+
+static gpointer _win32_name_lookup_thread(gpointer data) {
+	gchar *fullname = NULL;
+	wchar_t username[UNLEN + 1];
+	DWORD dwLenUsername = UNLEN + 1;
+
+	GetUserNameW((LPWSTR) &username, &dwLenUsername);
+
+	if (username != NULL && *username != '\0') {
+		LPBYTE servername = NULL;
+		LPBYTE info = NULL;
+
+		NetGetDCName(NULL, NULL, &servername);
+
+		/* purple_debug_info("bonjour", "Looking up the full name from the %s.\n", (servername ? "domain controller" : "local machine")); */
+
+		if (NetUserGetInfo((LPCWSTR) servername, username, 10, &info) == NERR_Success
+				&& info != NULL && ((LPUSER_INFO_10) info)->usri10_full_name != NULL
+				&& *(((LPUSER_INFO_10) info)->usri10_full_name) != '\0') {
+			fullname = g_utf16_to_utf8(
+				((LPUSER_INFO_10) info)->usri10_full_name,
+				-1, NULL, NULL, NULL);
+		}
+		/* Fall back to the local machine if we didn't get the full name from the domain controller */
+		else if (servername != NULL) {
+			/* purple_debug_info("bonjour", "Looking up the full name from the local machine"); */
+
+			if (info != NULL) NetApiBufferFree(info);
+			info = NULL;
+
+			if (NetUserGetInfo(NULL, username, 10, &info) == NERR_Success
+					&& info != NULL && ((LPUSER_INFO_10) info)->usri10_full_name != NULL
+					&& *(((LPUSER_INFO_10) info)->usri10_full_name) != '\0') {
+				fullname = g_utf16_to_utf8(
+					((LPUSER_INFO_10) info)->usri10_full_name,
+					-1, NULL, NULL, NULL);
+			}
+		}
+
+		if (info != NULL) NetApiBufferFree(info);
+		if (servername != NULL) NetApiBufferFree(servername);
+
+		if (!fullname)
+			fullname = g_utf16_to_utf8(username, -1, NULL, NULL, NULL);
+	}
+
+	purple_timeout_add(0, _set_default_name_cb, fullname);
+
+	return NULL;
+}
+#endif
+
 static void
 initialize_default_account_values()
 {
-#ifdef _WIN32
-	char *fullname = NULL;
-#else
+#ifndef _WIN32
 	struct passwd *info;
-	const char *fullname = NULL;
 #endif
-	char *splitpoint = NULL;
-	char *tmp;
+	const char *fullname = NULL, *splitpoint, *tmp;
 	char hostname[255];
+	gchar *conv = NULL;
 
 #ifndef _WIN32
 	/* Try to figure out the user's real name */
@@ -464,84 +606,29 @@ initialize_default_account_values()
 		fullname = info->pw_gecos;
 	else if ((info != NULL) && (info->pw_name != NULL) && (info->pw_name[0] != '\0'))
 		fullname = info->pw_name;
-	else if (((fullname = getlogin()) != NULL) && (fullname[0] != '\0'))
-		;
-	else
-		fullname = _("Purple Person");
-	/* Make sure fullname is valid UTF-8.  If not, try to convert it. */
-	if (!g_utf8_validate(fullname, -1, NULL))
-	{
-		gchar *tmp;
-		tmp = g_locale_to_utf8(fullname, -1, NULL, NULL, NULL);
-		if ((tmp == NULL) || (*tmp == '\0'))
-			fullname = _("Purple Person");
-	}
-
+	else if (((fullname = getlogin()) != NULL) && (fullname[0] == '\0'))
+		fullname = NULL;
 #else
-	FARPROC myNetUserGetInfo = wpurple_find_and_loadproc("Netapi32.dll",
-		"NetUserGetInfo");
+	/* The Win32 username lookup functions are synchronous so we do it in a thread */
+	g_thread_create(_win32_name_lookup_thread, NULL, FALSE, NULL);
+#endif
 
-	if (myNetUserGetInfo) {
-		LPUSER_INFO_10 user_info = NULL;
-		LPSERVER_INFO_100 server_info = NULL;
-		wchar_t *servername = NULL;
-		wchar_t username[UNLEN + 1];
-		DWORD dwLenUsername = UNLEN + 1;
-		FARPROC myNetServerEnum = wpurple_find_and_loadproc(
-			"Netapi32.dll", "NetServerEnum");
-		FARPROC myNetApiBufferFree = wpurple_find_and_loadproc(
-			"Netapi32.dll", "NetApiBufferFree");
-
-		if (myNetServerEnum && myNetApiBufferFree) {
-			DWORD dwEntriesRead = 0;
-			DWORD dwTotalEntries = 0;
-			DWORD dwResumeHandle = 0;
-
-			NET_API_STATUS nStatus = (myNetServerEnum)(NULL, 100,
-				&server_info, MAX_PREFERRED_LENGTH,
-				&dwEntriesRead, &dwTotalEntries,
-				SV_TYPE_DOMAIN_CTRL, NULL, &dwResumeHandle);
-
-			if ((nStatus == NERR_Success
-					|| nStatus == ERROR_MORE_DATA)
-					&& dwEntriesRead > 0) {
-				servername = server_info->sv100_name;
-			} else {
-				purple_debug_warning("bonjour", "Unable to look up domain controller. NET_API_STATUS = %d, Entries Read = %d, Total Entries = %d\n", nStatus, dwEntriesRead, dwTotalEntries);
-			}
-		}
-
-		if (!GetUserNameW((LPWSTR) &username, &dwLenUsername)) {
-			purple_debug_warning("bonjour",
-				"Unable to look up username\n");
-		}
-
-		if (username != NULL && *username != '\0'
-				&& (myNetUserGetInfo)(servername, username, 10,
-					&user_info) == NERR_Success) {
-			if (user_info != NULL) {
-				fullname = g_utf16_to_utf8(
-					user_info->usri10_full_name,
-					-1, NULL, NULL, NULL);
-			}
-		}
-		if (user_info != NULL)
-			(myNetApiBufferFree)(user_info);
-		if (server_info != NULL)
-			(myNetApiBufferFree)(server_info);
+	/* Make sure fullname is valid UTF-8.  If not, try to convert it. */
+	if (fullname != NULL && !g_utf8_validate(fullname, -1, NULL)) {
+		fullname = conv = g_locale_to_utf8(fullname, -1, NULL, NULL, NULL);
+		if (conv == NULL || *conv == '\0')
+			fullname = NULL;
 	}
 
-	if (!fullname)
-		fullname = g_strdup(_("Purple Person"));
-#endif
+	if (fullname == NULL)
+		fullname = _("Purple Person");
 
 	/* Split the real name into a first and last name */
 	splitpoint = strchr(fullname, ' ');
-	if (splitpoint != NULL)
-	{
+	if (splitpoint != NULL) {
 		default_firstname = g_strndup(fullname, splitpoint - fullname);
 		tmp = &splitpoint[1];
-		
+
 		/* The last name may be followed by a comma and additional data.
 		 * Only use the last name itself.
 		 */
@@ -550,16 +637,12 @@ initialize_default_account_values()
 			default_lastname = g_strndup(tmp, splitpoint - tmp);
 		else
 			default_lastname = g_strdup(tmp);
-	}
-	else
-	{
+	} else {
 		default_firstname = g_strdup(fullname);
 		default_lastname = g_strdup("");
 	}
 
-#ifdef _WIN32
-	g_free(fullname);
-#endif
+	g_free(conv);
 
 	/* Try to figure out a good host name to use */
 	/* TODO: Avoid 'localhost,' if possible */
