@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include "msn.h"
 #include "msn-utils.h"
@@ -412,10 +412,9 @@ msn_import_html(const char *html, char **attributes, char **message)
 	*attributes = g_strdup_printf("FN=%s; EF=%s; CO=%s; PF=0; RL=%c",
 								  encode_spaces(fontface),
 								  fonteffect, fontcolor, direction);
-	*message = g_strdup(msg);
+	*message = msg;
 
 	g_free(fontface);
-	g_free(msg);
 }
 
 void
