@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02111-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -70,8 +70,9 @@ version_fetch_cb(PurpleUtilFetchUrlData *url_data, gpointer user_data,
 	message = g_string_new("");
 	g_string_append_printf(message, _("You are using %s version %s.  The "
 			"current version is %s.  You can get it from "
-			"<a href=\"" PURPLE_WEBSITE "\">" PURPLE_WEBSITE "</a><hr>"),
-			PIDGIN_NAME, purple_core_get_version(), cur_ver);
+			"<a href=\"%s\">%s</a><hr>"),
+			PIDGIN_NAME, purple_core_get_version(), cur_ver,
+			PURPLE_WEBSITE, PURPLE_WEBSITE);
 
 	if(*changelog) {
 		formatted = purple_strdup_withhtml(changelog);
